/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"
#include "TaskItem.h"
#include "PopupMenu.h"
#include "../lsapi/lsapi.h"


BOOL TaskItem::m_bWindowCaption = FALSE;
FARPROC (__stdcall *SwitchToThisWindow)(HWND, int);

TaskItem::TaskItem(HWND hWnd) : CommandItem(NULL, NULL, NULL)
{
	char pszWindowText[1024];
	GetWindowText(hWnd, pszWindowText, sizeof(pszWindowText));
	m_pszTitle = pszWindowText != NULL ? strdup(pszWindowText) : NULL;
	m_hWnd = hWnd;

	SwitchToThisWindow = (FARPROC (__stdcall *)(HWND, int))GetProcAddress(GetModuleHandle("USER32.DLL"), "SwitchToThisWindow");
}

TaskItem::~TaskItem()
{
	// do not destroy the icon!
	m_hIcon = NULL;

	if (m_pszTitle)
		free(m_pszTitle);
	m_pszTitle = NULL;
}

void TaskItem::Invoke()
{
	m_pParent->Hide(HIDE_PARENTS);

	if (SwitchToThisWindow)
	{
		if (IsIconic(m_hWnd))
			SwitchToThisWindow(m_hWnd, TRUE);
		else
			if (!SendMessage(GetLitestepWnd(), LM_BRINGTOFRONT, 0, (LPARAM)m_hWnd))
				SwitchToThisWindow(m_hWnd, TRUE);
	}
	ShowWindow (m_hWnd, SW_SHOWNORMAL); // in case it's hidden <g>
}

void TaskItem::Mouse(int nMsg, int x, int y, WPARAM wParam)
{

	char modkey = 0x00; // bits in human readable format
	/*
	least significant bit(
	MK_CONTROL The CTRL key is down. 
	MK_LBUTTON The left mouse button is down. 
	MK_MBUTTON The middle mouse button is down. 
	MK_RBUTTON The right mouse button is down. 
	MK_SHIFT The SHIFT key is down. 
	MK_XBUTTON1 Windows 2000/XP: The first X button is down. 
	MK_XBUTTON2 Windows 2000/XP: The second X button is down. ) most significant bit
	0x00: no modkey
	0x01: CTRL
	0x02: LBUTTON
	0x08: RBUTTON
	0x10: SHIFT
	0x11: CTRL + SHIFT
	
	*/

	if ((LOWORD(wParam) & (MK_CONTROL|MK_SHIFT)) == (MK_CONTROL|MK_SHIFT))
		modkey = 0x11;
	else if ((LOWORD(wParam) & MK_CONTROL) == MK_CONTROL)
		modkey = 0x01;
	else if ((LOWORD(wParam) & MK_SHIFT) == MK_SHIFT)
		modkey = 0x10;
	else if ((LOWORD(wParam) & MK_RBUTTON) == MK_RBUTTON)
		modkey = 0x08;
	else modkey = 0x00;

	// CTRL is modkey
	if (0x01 == modkey)
	{
		if ( (nMsg == WM_LBUTTONUP) || (nMsg == WM_RBUTTONUP) )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// hide this mofo!
			if (PtInRect(&r, p))
			{
				//PostMessage(m_hWnd,WM_WINDOWPOSCHANGING,0,0)
				SetWindowPos(m_hWnd, NULL, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_HIDEWINDOW
					| SWP_NOREDRAW | SWP_NOACTIVATE | SWP_NOZORDER);
			}
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}

	// R-button is modifier 
	// works the other way around, apparently, looking for a fix  - normalperson
	else if (0x08 == modkey)
	{

		if (nMsg == WM_LBUTTONUP )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			if (PtInRect(&r, p))
			{
				SetForegroundWindow(GetLastActivePopup(m_hWnd));
				PostMessage(m_hWnd, 0x313, 0, GetMessagePos() );
			}
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}

	// SHIFT key is modifier
	else if (0x10 == modkey)
	{
		if ( (nMsg == WM_LBUTTONUP) || (nMsg == WM_RBUTTONUP) )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// close this mofo!
			if (PtInRect(&r, p))
				PostMessage(m_hWnd,WM_CLOSE,0,0);
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}

	// CTRL+SHIFT key modifier
	else if (0x11 == modkey)
	{
		if ( (nMsg == WM_LBUTTONUP) || (nMsg == WM_RBUTTONUP) )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);

			// KILL this mofo!
			if (PtInRect(&r, p))
			{
				DWORD ProcID;
				GetWindowThreadProcessId(m_hWnd,&ProcID);
				TerminateProcess(OpenProcess(PROCESS_ALL_ACCESS, TRUE, ProcID), 0);
			}
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);

	}

	// no modkey
	else if (0x00 == modkey)
	{
		if ( (nMsg == WM_RBUTTONUP ) || (nMsg == WM_LBUTTONUP) )
		{
			RECT r;
			POINT p;
			p.x = x;
			p.y = y;
			GetItemRect(&r);
			if (PtInRect(&r, p))
			{
				//SetWindowPos(m_hWnd, NULL, 50, 50, 0, 0, SWP_NOSIZE);
				Invoke();
			}
		}
		else CommandItem::Mouse(nMsg, x, y, wParam);
	}
	else CommandItem::Mouse(nMsg, x, y, wParam);
}

void TaskItem::Paint(HDC hDC)
{
	if (m_bWindowCaption)
	{
		RECT r;
		GetItemRect(&r);

		if (m_bActive)
			DrawCaption(m_hWnd, hDC, &r, DC_ACTIVE | DC_ICON | DC_TEXT);
		else
			DrawCaption(m_hWnd, hDC, &r, DC_ICON | DC_TEXT);

		if (m_bDrawBevel)
			PaintBevel(hDC, m_nTop);
	}
	else
	{
		CommandItem::Paint(hDC);
	}

}
